@@ -52,6 +52,7 @@
 #import <string.h>
 #import <libc.h>
 #import <dns_util.h>
+#import <dns_private.h>
 #import <ifaddrs.h>
 
 #define DNS_FLAGS_QR_MASK  0x8000
@@ -86,8 +87,6 @@ extern void __dns_open_notify();
 
 static const char hexchar[] = "0123456789abcdef";
 
-#ifdef DEBUG
-
 static char *_log_status_string(dns_reply_t *r)
 {
 	static char str[64];
@@ -165,13 +164,13 @@ log_reply(dns_reply_t *r)
 
 	if (r == NULL)
 	{
-		syslog(LOG_ERR, "*** NULL DNS REPLY ***");
+		system_log(LOG_ERR, "*** NULL DNS REPLY ***");
 		return;
 	}
 
 	h = r->header;
 
-	syslog(LOG_ERR, "*** DNS: %s/%u/%s/%s/%s/%s%s%s%s/%s/%hu %hu %hu %hu",
+	system_log(LOG_ERR, "*** DNS: %s/%u/%s/%s/%s/%s%s%s%s/%s/%hu %hu %hu %hu",
 		_log_server_string(r->server), h->xid, _log_status_string(r), _log_qr_string(h), _log_opcode_string(h),
 		_log_bool(h->flags & DNS_FLAGS_AA), _log_bool(h->flags & DNS_FLAGS_TC), 
 		_log_bool(h->flags & DNS_FLAGS_RD), _log_bool(h->flags & DNS_FLAGS_RA),
@@ -182,18 +181,7 @@ log_reply(dns_reply_t *r)
 void
 log_query(char *name, u_int32_t class, u_int32_t type)
 {
-	syslog(LOG_ERR, "??? DNS: %s %s %s", name, dns_class_string(class), dns_type_string(type));
-}
-#endif DEBUG
-
-int
-msg_callback(int priority, char *msg)
-{
-#ifdef DEBUG
-	priority = LOG_ERR;
-#endif
-	system_log(priority, msg);
-	return 0;
+	system_log(LOG_ERR, "??? DNS: %s %s %s", name, dns_class_string(class), dns_type_string(type));
 }
 
 static char *
@@ -246,7 +234,7 @@ _dns_inet_ntop(struct in6_addr a, u_int32_t iface)
 		}
 
 		x = iface;
-		s6.sin6_addr.__u6_addr.__u6_addr16[1] = x;
+		s6.sin6_addr.__u6_addr.__u6_addr16[1] = htons(x);
 	}
 
 	inet_ntop(AF_INET6, p, buf, 128);
@@ -268,8 +256,8 @@ reverse_ipv4(char *v4)
 	if (v4 == NULL) return NULL;
 
 	if (inet_aton(v4, &s) == 0) return NULL;
-	
-	ab.a = ntohl(s.s_addr);
+
+	ab.a = s.s_addr;
 
 	asprintf(&p, "%u.%u.%u.%u.in-addr.arpa.", ab.b[3], ab.b[2], ab.b[1], ab.b[0]);
 	return p;
@@ -365,7 +353,7 @@ alt_ntoa(int32_t alt)
 
 	frac = alt % 100;
 	m = (alt / 100) * sign;
-	
+
 	asprintf(&p, "%d.%.2d", m, frac);
 	return p;
 }
@@ -406,7 +394,7 @@ precsize_ntoa(u_int8_t prec)
 	DNSAgent *agent;
 
 	agent = [super alloc];
-	system_log(LOG_DEBUG, "Allocated DNSAgent 0x%08x\n", (int)agent);
+	system_log(LOG_DEBUG, "Allocated DNSAgent 0x%08x", (int)agent);
 	return agent;
 }
 
@@ -418,13 +406,19 @@ precsize_ntoa(u_int8_t prec)
 
 	config = [configManager configForAgent:"DNSAgent" fromConfig:[configManager config]];
 	buffersize = [configManager intForKey:"BufferSize" dict:config default:DNS_DEFAULT_BUFFER_SIZE];
+	dnsDebug = [configManager boolForKey:"Debug" dict:config default:NO];
+
+	if (agent_debug_enabled) dnsDebug = YES;
 
 	syslock_lock(rpcLock);
 	/* DNS API */
 	__dns_open_notify();
 	dns = dns_open(NULL);
+	if ((dns != NULL) && (dns->sdns != NULL) && aaaa_cutoff_enabled) dns->sdns->flags |= DNS_FLAG_OK_TO_SKIP_AAAA;
 	syslock_unlock(rpcLock);
 
+	if (dnsDebug) dns_set_debug(dns, 1);
+
 	dns_set_buffer_size(dns, buffersize);
 
 	if (dns == NULL)
@@ -452,7 +446,7 @@ precsize_ntoa(u_int8_t prec)
 	if (stats != nil) [stats release];
 	/* DNS API */
 	dns_free(dns);
-	system_log(LOG_DEBUG, "Deallocated DNSAgent 0x%08x\n", (int)self);
+	system_log(LOG_DEBUG, "Deallocated DNSAgent 0x%08x", (int)self);
 	[super dealloc];
 }
 
@@ -589,7 +583,7 @@ precsize_ntoa(u_int8_t prec)
 			[item setValue:str forKey:"preference"];
 			[item setValue:r->data.MX->name forKey:"mail_exchanger"];
 			break;
-			
+
 		case ns_t_txt:
 			[item setValue:r->name forKey:"name"];
 			for (i=0; i<r->data.TXT->string_count; i++)
@@ -817,7 +811,7 @@ precsize_ntoa(u_int8_t prec)
 		 */
 		if (cname_count > 0) realName = lowerCase([host valueForKey:"cname"]);
 		else realName = lowerCase(r->answer[0]->name);
-		
+
 		[host mergeValue:realName forKey:"name"];
 	}
 
@@ -831,7 +825,7 @@ precsize_ntoa(u_int8_t prec)
 	{
 		domainName = postfix(realName, '.');
 		if (domainName != NULL) [host setValue:domainName forKey:"_lookup_DNS_domain"];
-	
+
 		freeString(realName);
 		freeString(domainName);
 	}
@@ -859,13 +853,14 @@ precsize_ntoa(u_int8_t prec)
 	if (k == NULL) return nil;
 
 	/* DNS API */
-#ifdef DEBUG
-	log_query(k, ns_c_in, t);
-#endif
+
+	if (dnsDebug) dns_set_debug(dns, 1);
+	else dns_set_debug(dns, 0);
+
+	if (dnsDebug) log_query(k, ns_c_in, t);
 	r = dns_lookup(dns, k, ns_c_in, t);
-#ifdef DEBUG
-	log_reply(r);
-#endif
+	if (dnsDebug) log_reply(r);
+
 	host = [self dictForDNSReply:r];
 	dns_free_reply(r);
 
@@ -889,13 +884,13 @@ precsize_ntoa(u_int8_t prec)
 		ab.b[3], ab.b[2], ab.b[1], ab.b[0]);
 
 	/* DNS API */
-#ifdef DEBUG
-	log_query(name, ns_c_in, ns_t_ptr);
-#endif
+	if (dnsDebug) dns_set_debug(dns, 1);
+	else dns_set_debug(dns, 0);
+
+	if (dnsDebug) log_query(name, ns_c_in, ns_t_ptr);
 	r = dns_lookup(dns, name, ns_c_in, ns_t_ptr);
-#ifdef DEBUG
-	log_reply(r);
-#endif
+	if (dnsDebug) log_reply(r);
+
 	host = [self dictForDNSReply:r];
 	dns_free_reply(r);
 
@@ -916,13 +911,13 @@ precsize_ntoa(u_int8_t prec)
 	char str[64];
 
 	/* DNS API */
-#ifdef DEBUG
-	log_query(name, ns_c_in, ns_t_ptr);
-#endif
+	if (dnsDebug) dns_set_debug(dns, 1);
+	else dns_set_debug(dns, 0);
+
+	if (dnsDebug) log_query(name, ns_c_in, ns_t_ptr);
 	r = dns_lookup(dns, name, ns_c_in, ns_t_ptr);
-#ifdef DEBUG
-	log_reply(r);
-#endif
+	if (dnsDebug) log_reply(r);
+
 	net = [self dictForDNSReply:r];
 	dns_free_reply(r);
 
@@ -944,7 +939,7 @@ precsize_ntoa(u_int8_t prec)
 
 	snprintf(str, 64, "%s.%s.%s.%s", l[3], l[2], l[1], l[0]);
 	freeList(l);
-	
+
 	longName = [net valueForKey:"ptr_name"];
 	shortName = prefix(longName, '.');
 	domainName = postfix(longName, '.');
@@ -980,13 +975,13 @@ precsize_ntoa(u_int8_t prec)
 		ab.b[3], ab.b[2], ab.b[1], ab.b[0]);
 
 	/* DNS API */
-#ifdef DEBUG
-	log_query(name, ns_c_in, ns_t_ptr);
-#endif
+	if (dnsDebug) dns_set_debug(dns, 1);
+	else dns_set_debug(dns, 0);
+
+	if (dnsDebug) log_query(name, ns_c_in, ns_t_ptr);
 	r = dns_lookup(dns, name, ns_c_in, ns_t_ptr);
-#ifdef DEBUG
-	log_reply(r);
-#endif
+	if (dnsDebug) log_reply(r);
+
 	net = [self dictForDNSReply:r];
 	dns_free_reply(r);
 
@@ -1142,13 +1137,13 @@ precsize_ntoa(u_int8_t prec)
 	int i, len;
 
 	/* SDNS API */
-#ifdef DEBUG
-	log_query(q->name, q->class, q->type);
-#endif
+	if (dnsDebug) dns_set_debug(dns, 1);
+	else dns_set_debug(dns, 0);
+
+	if (dnsDebug) log_query(q->name, q->dnsclass, q->dnstype);
 	r = dns_lookup(dns, q->name, q->dnsclass, q->dnstype);
-#ifdef DEBUG
-	log_reply(r);
-#endif
+	if (dnsDebug) log_reply(r);
+
 	sub = [self arrayForDNSReply:r];
 
 	dns_free_reply(r);
@@ -1176,8 +1171,8 @@ precsize_ntoa(u_int8_t prec)
 
 	if (cat != LUCategoryHost) return nil;
 	if (pattern == nil) return nil;
-	
-	q.dnsclass = ns_c_in;	
+
+	q.dnsclass = ns_c_in;
 
 	/*
 	 * search will be based on one of the following keys
@@ -1220,11 +1215,11 @@ precsize_ntoa(u_int8_t prec)
 	if (name != NULL)
 	{
 		if ((protocol != NULL) && (service == NULL)) return nil;
-		
+
 		if (service != NULL)
 		{
 			if (protocol == NULL) return nil;
-			
+
 #ifdef RFC_2782
 			asprintf(&qname, "_%s._%s.%s", service, protocol, name);
 #else
@@ -1292,7 +1287,7 @@ precsize_ntoa(u_int8_t prec)
 		if (fixup & FIX_IPV4) [item mergeValue:ip forKey:"ip_address"];
 		if (fixup & FIX_IPV6) [item mergeValue:ipv6 forKey:"ipv6_address"];
 	}
-	
+
 	list = [all filter:pattern];
 	[all release];
 
@@ -1328,7 +1323,7 @@ precsize_ntoa(u_int8_t prec)
 	type = atoi(ctype);
 
 	do_search = [dict unsignedLongForKey:"search"];
-	
+
 	fromlen = sizeof(struct sockaddr_storage);
 	from = (struct sockaddr *)calloc(1, fromlen);
 
@@ -1373,7 +1368,7 @@ precsize_ntoa(u_int8_t prec)
 
 	[item setValue:buf forKey:"server"];
 
-#ifdef DEBUG
+	if (dnsDebug) 
 	{
 		char *d_buf;
 		dns_reply_t *d_r;
@@ -1400,7 +1395,6 @@ precsize_ntoa(u_int8_t prec)
 
 		free(d_buf);
 	}
-#endif
 
 	free(buf);
 	free(b64);
