@@ -48,11 +48,13 @@
 #import "_lu_types.h"
 #import <netdb.h>
 #import <sys/socket.h>
+#import <sys/param.h>
 #import <netinet/in.h>
 #import <arpa/inet.h>
 #import <strings.h>
 #import <NetInfo/dsutil.h>
 #import <stdio.h>
+#import <time.h>
 
 /* 2 second timeout on sends */
 #define TIMEOUT_MSECONDS (2000)
@@ -92,7 +94,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryUser;
-	
+
 	/*
 	 * getpwent-A (returns BSD4.4 data)
 	 */
@@ -101,7 +103,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryUser;
-	
+
 	/*
 	 * getpwuid (returns BSD4.3 data)
 	 */
@@ -110,7 +112,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valInt;
 	proc_helper[i].key = "uid";
 	proc_helper[i].cat = LUCategoryUser;
-	
+
 	/*
 	 * getpwuid_A (returns BSD4.4 data)
 	 */
@@ -128,7 +130,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valString;
 	proc_helper[i].key = "name";
 	proc_helper[i].cat = LUCategoryUser;
-	
+
 	/*
 	 * getpwname_A (returns BSD4.4 data)
 	 */
@@ -146,7 +148,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryGroup;
-	
+
 	/*
 	 * getgrgid
 	 */
@@ -174,7 +176,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryHost;
 	proc_helper[i].encoder = @selector(encodeHost:intoXdr:);
-	
+
 	/*
 	 * gethostbyname
 	 */
@@ -210,7 +212,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryNetwork;
-	
+
 	/*
 	 * getnetbyname
 	 */
@@ -237,7 +239,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryService;
-	
+
 	/*
 	 * getprotoent
 	 */
@@ -246,7 +248,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryProtocol;
-	
+
 	/*
 	 * getprotobyname
 	 */
@@ -273,7 +275,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryRpc;
-	
+
 	/*
 	 * getrpcbyname
 	 */
@@ -301,7 +303,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryMount;
-	
+
 	/*
 	 * getfsbyname
 	 */
@@ -320,7 +322,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryPrinter;
-	
+
 	/*
 	 * grdb_getbyname
 	 */
@@ -338,7 +340,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryBootparam;
-	
+
 	/*
 	 * bootparams_getbyname
 	 */
@@ -347,7 +349,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valString;
 	proc_helper[i].key = "name";
 	proc_helper[i].cat = LUCategoryBootparam;
-	
+
 	/*
 	 * bootp_getbyip
 	 */
@@ -374,7 +376,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valString;
 	proc_helper[i].key = "name";
 	proc_helper[i].cat = LUCategoryAlias;
-	
+
 	/*
 	 * alias_getent
 	 */
@@ -383,7 +385,7 @@ extern char *nettoa(u_int32_t net);
 	proc_helper[i].decoder = valNull;
 	proc_helper[i].key = NULL;
 	proc_helper[i].cat = LUCategoryAlias;
-	
+
 	return self;
 }
 
@@ -406,7 +408,7 @@ extern char *nettoa(u_int32_t net);
 	t = [Thread currentThread];
 	request = (lookup_request_msg *)[t data];
 	[t setData:NULL];
-	
+
 	/*
 	 * Use the MIG server to dispatch messages.
 	 * Server functions for the MIG interface are in lookup_proc.m
@@ -472,6 +474,8 @@ extern char *nettoa(u_int32_t net);
 	BOOL test;
 	char logString[512];
 	SEL aSel;
+	time_t tick;
+	struct tm gtime;
 
 	t = [Thread currentThread];
 	sprintf(logString, "%s", proc_name(procno));
@@ -548,7 +552,7 @@ extern char *nettoa(u_int32_t net);
 			free(val);
 			val = NULL;
 		}
-	
+
 		if (val == NULL)
 		{
 			system_log(LOG_DEBUG, "%s - can't decode lookup value", logString);
@@ -674,7 +678,7 @@ extern char *nettoa(u_int32_t net);
 				system_log(LOG_DEBUG, "%s - can't decode lookup value", logString);
 				return NO;
 			}
-	
+
 			system_log(LOG_DEBUG, "%s %s", logString, name);
 			if (streq(name, "config"))
 			{
@@ -798,7 +802,7 @@ extern char *nettoa(u_int32_t net);
 			name = NULL;
 			[self xdrInt:(test ? 1 : 0) buffer:outdata length:outlen];
 			return YES;
-	
+
 		case PROC_CHECKNETWAREENBL: /* NONSTANDARD */
 			system_log(LOG_DEBUG, "%s", logString);
 			test = [server isNetwareEnabled];
@@ -823,6 +827,11 @@ extern char *nettoa(u_int32_t net);
 			if (statistics == NULL) return NO;
 			sprintf(logString, "%u", [rover totalMemory]);
 			[statistics setValue:logString forKey:"# Total Memory"];
+			tick = time(NULL);
+			gmtime_r(&tick, &gtime);
+			sprintf(logString, "%d.%02d.%02d %02d:%02d:%02d UTC", gtime.tm_year + 1900, gtime.tm_mon + 1, gtime.tm_mday, gtime.tm_hour, gtime.tm_min, gtime.tm_sec);
+			[statistics setValue:logString forKey:"# Timestamp"];
+
 			test = [self xdrItem:statistics method:@selector(encodeDictionary:intoXdr:) buffer:outdata length:outlen];
 			return test;
 
@@ -957,7 +966,7 @@ extern char *nettoa(u_int32_t net);
 	{
 		strcat(opts, optsList[i]);
 		if (i < (count - 1)) strcat(opts, ",");
-	
+
 		if ((streq(optsList[i], "rw")) ||
 			(streq(optsList[i], "rq")) ||
 			(streq(optsList[i], "ro")) ||
@@ -1082,7 +1091,7 @@ extern char *nettoa(u_int32_t net);
 {
 	lu_xdr_t *outxdr;
 	int32_t status;
-	
+
 	outxdr = lu_xdr_alloc(0, 0);
 
 	status = lu_xdr_int_32(outxdr, &i);
@@ -1112,7 +1121,7 @@ extern char *nettoa(u_int32_t net);
 	int32_t status;
 
 	if (list == nil) return NO;
-	
+
 	outxdr = lu_xdr_alloc(0, 0);
 	count = [list count];
 
@@ -1156,7 +1165,7 @@ extern char *nettoa(u_int32_t net);
 	lu_xdr_int_32(outxdr, &count);
 
 	if (count == 0)
-	{		
+	{
 		*len = lu_xdr_getpos(outxdr);
 		*data = outxdr->buf;
 		free(outxdr);
@@ -1309,7 +1318,7 @@ extern char *nettoa(u_int32_t net);
 
 	lu_xdr_free(inxdr);
 
-	ip.s_addr = i;
+	ip.s_addr = htonl(i);
 	str = malloc(16);
 	if (inet_ntop(AF_INET, &ip, str, 16) == NULL)
 	{
@@ -1571,7 +1580,7 @@ extern char *nettoa(u_int32_t net);
 		return NULL;
 	}
 
-	return item;	
+	return item;
 }
 
 - (char **)intAndStringFromBuffer:(char *)buf length:(int)len
