@@ -32,6 +32,7 @@
 #include <netinfo/ni.h>
 #include <string.h>
 #include <stdlib.h>
+#include <notify.h>
 #include <sys/param.h>
 #include <sys/dir.h>
 #include "ni_file.h"
@@ -647,6 +648,7 @@ file_init(char *rootdir, void **hdl)
 	if (size == 0) doconvert = 0;
 
 	flags = DSSTORE_FLAGS_ACCESS_READWRITE;
+	flags |= DSSTORE_FLAGS_NOTIFY_CHANGES;
 	if (!i_am_clone) flags |= DSSTORE_FLAGS_SERVER_MASTER;
 
 	status = dsstore_new(&s, rootdir, flags);
@@ -756,6 +758,7 @@ ni_status file_idunalloc(void *hdl, ni_id id)
 
 	if (hdl == NULL) return NI_SYSTEMERR;
 	status = dsstore_remove(STORE(hdl), id.nii_object);
+
 	return dstonistatus(status);
 }
 
@@ -836,7 +839,6 @@ file_write(void *hdl, ni_object *obj)
 	}
 
 	dsrecord_release(r);
-
 	return dstonistatus(status);
 }
 
@@ -890,7 +892,15 @@ file_store_version(void* hdl)
 	return dsstore_version(STORE(hdl));
 }
 
-ni_index file_version(void *hdl, ni_id id)
+ni_index
+file_version(void *hdl, ni_id id)
 {
 	return dsstore_record_version(STORE(hdl), id.nii_object);
 }
+
+void
+file_notify(void *hdl)
+{
+	dsstore_notify(STORE(hdl));
+}
+
