@@ -59,9 +59,7 @@
 #include <sys/stat.h>
 #include <rpc/rpc.h>
 #include <rpc/pmap_clnt.h>
-
-#include <bsm/libbsm.h>
-#include <bsm/audit_uevents.h>
+#include <rpc/pmap_prot.h>
 
 #ifdef _OS_NEXT_
 /* Support the old rpcgen, which doesn't append the _svc suffix
@@ -124,12 +122,12 @@ CLIENT *svctcp_getclnt(SVCXPRT *xprt);
 #define RPARENT_CATCH_SLEEP_SECONDS 2
 
 /*
- * Ping timeout is 1/2 second per try, with 10 tries.  Total timeout is 5 seconds.
+ * Ping timeout is 1/2 second per try, with 3 tries.  Total timeout is 2 seconds.
  */
  
 #define PING_TIMEOUT_SEC  0 
 #define PING_TIMEOUT_USEC 500000
-#define PING_CALL_SEC  5 
+#define PING_CALL_SEC  2 
 #define PING_CALL_USEC 0
 
 /* Number of minutes to force between attempts to see if we are still root */
@@ -169,13 +167,13 @@ typedef enum
 	TokenXMLDictEnd = 5,
 	TokenXMLKeyStart = 6,
 	TokenXMLKeyEnd = 7
-} xml_token_type_t;
+} token_type_t;
 
 typedef struct
 {
-	xml_token_type_t type;
+	token_type_t type;
 	char *value;
-} xml_token_t;
+} token_t;
 
 /*
  * Is this call an update from the master?
@@ -252,40 +250,6 @@ is_admin(void *ni, char *user)
 	return FALSE;
 }
 
-static void
-audit_auth(uid_t uid, char *msg, int status, int err)
-{
-	int aufd;
-	token_t *tok;
-	auditinfo_t auinfo;
-	long au_cond;
-    
-	/* If we are not auditing, don't cut an audit record; just return */
-	if (auditon(A_GETCOND, &au_cond, sizeof(long)) < 0) return;
-	if (au_cond == AUC_NOAUDIT) return;
-	if (getaudit(&auinfo) != 0) return;
-	if ((aufd = au_open()) == -1) return;
-    
-	/* The uid being authenticated */
-    tok = au_to_subject32(auinfo.ai_auid, 0, 0, uid, -1, getpid(), auinfo.ai_asid, &auinfo.ai_termid);
-    if (tok == NULL) return;
-	au_write(aufd, tok);
-
-    /* Message text */
-    if (msg != NULL)
-    {
-        tok = au_to_text(msg);
-        if (tok != NULL) au_write(aufd, tok);
-    }
-
-    /* Status and errno */
-    tok = au_to_return32(status, err);
-	if (tok != NULL) au_write(aufd, tok);
-    
-	au_close(aufd, 1, AUE_auth_user);
-	return;
-}
-
 /*
  * Authenticate a NetInfo call. Only required for write operations.
  * NetInfo uses passwords for authentications, but does not send them
@@ -343,8 +307,6 @@ authenticate(void *ni, struct svc_req *req)
 	status = ni_lookup(ni, &id, NAME_NAME, NAME_USERS, &idl);
 	if (status != NI_OK)
 	{
-        audit_auth(aup->aup_uid, "can't access /users", 1, ENOENT);
-
 		system_log(LOG_ERR,
 				"Cannot authenticate user %d from %s:%hu - no /%s "
 				"directory: %s", aup->aup_uid,
@@ -366,7 +328,6 @@ authenticate(void *ni, struct svc_req *req)
 	status = ni_lookup(ni, &id, NAME_UID, uidstr, &idl);
 	if (status != NI_OK)
 	{
-        audit_auth(aup->aup_uid, "no such user id", 1, ENOENT);
 		system_log(LOG_ERR, "Cannot find user %d from %s:%hu: %s",
 				aup->aup_uid,
 				inet_ntoa(sin->sin_addr), ntohs(sin->sin_port),
@@ -478,7 +439,6 @@ authenticate(void *ni, struct svc_req *req)
 		/*
 		 * No user with this uid with no password or a matching password
 		 */
-        audit_auth(aup->aup_uid, "authentication failed", 1, EAUTH);
 		system_log(LOG_ERR, "Authentication error for user "
 				"%d from %s:%hu",
 				aup->aup_uid,
@@ -492,7 +452,6 @@ authenticate(void *ni, struct svc_req *req)
 	status = ni_lookupprop(ni, &id, NAME_NAME, &nl);
 	if (status != NI_OK)
 	{
-        audit_auth(aup->aup_uid, "uid with no name property", 1, EAUTH);
 		system_log(LOG_ERR,
 				"User %d from %s:%hu - name prop not found during "
 				"authentication",
@@ -505,7 +464,6 @@ authenticate(void *ni, struct svc_req *req)
 
 	if (nl.ni_namelist_len == 0)
 	{
-        audit_auth(aup->aup_uid, "uid with no name value", 1, EAUTH);
 		system_log(LOG_ERR,
 				"User %d from %s:%hu - name value not found during "
 				"authentication",
@@ -535,7 +493,6 @@ authenticate(void *ni, struct svc_req *req)
 	}
 
 	auth_count[GOOD]++;
-    audit_auth(aup->aup_uid, NULL, 0, 0);
 	system_log(LOG_NOTICE,
 		"Authenticated user %s [%d] from %s:%hu", 
 		nl.ni_namelist_val[0], aup->aup_uid,
@@ -1008,7 +965,7 @@ _ni_statistics_2_svc(
 #ifdef _OS_NEXT_
 	wPropsN(P_CLEANUP, STATS_CLEANUP_TOGO, "%ld", cleanupwait < 0 ? -1 : (cleanuptime - now.tv_sec)/60);
 #else
-	wPropsN(P_CLEANUP, STATS_CLEANUP_TOGO, "%d", cleanupwait < 0 ? -1 : (cleanuptime - now.tv_sec)/60);
+	wPropsN(P_CLEANUP, STATS_CLEANUP_TOGO, "%ld", cleanupwait < 0 ? -1 : (cleanuptime - now.tv_sec)/60);
 #endif
 
 	/* current binding status */
@@ -1899,7 +1856,7 @@ add_hardwired_binding(struct in_addr server_addr, ni_name server_tag, ni_name cl
 }
 
 static void
-freeToken(xml_token_t *t)
+freeToken(token_t *t)
 {
 	if (t == NULL) return;
 	if (t->value != NULL) free(t->value);
@@ -1912,15 +1869,15 @@ freeToken(xml_token_t *t)
  * XML tokens are any run enclosed by < and >.
  * White space includes spaces, tabs, and newlines.
  */
-static xml_token_t *
+static token_t *
 get_token_1(FILE *fp, int xword)
 {
-	xml_token_t *t, *s;
+	token_t *t, *s;
 	char c;
 	static char x = EOF;
 	int i, run, len, xml;
 
-	t = (xml_token_t *)malloc(sizeof(xml_token_t));
+	t = (token_t *)malloc(sizeof(token_t));
 	t->type = TokenNULL;
 	t->value = NULL;
 	len = 0;
@@ -1951,7 +1908,7 @@ get_token_1(FILE *fp, int xword)
 
 		if (t->value == NULL) return t;
 
-		s = (xml_token_t *)malloc(sizeof(xml_token_t));
+		s = (token_t *)malloc(sizeof(token_t));
 		s->type = TokenWord;
 		s->value = NULL;
 
@@ -2033,10 +1990,10 @@ get_token_1(FILE *fp, int xword)
 	return t;
 }	
 
-static xml_token_t *
+static token_t *
 pop_token(FILE *fp)
 {
-	xml_token_t *t, *s;
+	token_t *t, *s;
 		
 	t = get_token_1(fp, 0);
 	if (t->type != TokenWord) return t;
@@ -2094,7 +2051,7 @@ fread_plist(FILE *fp)
 {
 	ni_proplist *pl;
 	ni_property *p;
-	xml_token_t *t;
+	token_t *t;
 
 	t = pop_token(fp);
 
@@ -2452,19 +2409,19 @@ bind_to_parent(struct in_addr *addrs, ni_rparent_stuff *stuff, unsigned int nadd
 
 		if (sys_is_my_broadcast(&(addrs[i]))) continue;
 
-		if (ni_ping(addrs[i].s_addr, stuff->bindings[i].server_tag))
+		stat = ni_ping(addrs[i].s_addr, stuff->bindings[i].server_tag);
+		if (stat == RPC_SUCCESS)
 		{
 			ni_name_free(&(res->ni_rparent_res_u.binding.tag));
 			res->ni_rparent_res_u.binding.tag = ni_name_dup(stuff->bindings[i].server_tag);
 			res->ni_rparent_res_u.binding.addr = addrs[i].s_addr;
 
-			system_log(LOG_INFO, "Anonymous binding to %s/%s",
-				inet_ntoa(addrs[i]), stuff->bindings[i].server_tag);
+			system_log(LOG_INFO, "Anonymous binding to %s/%s", inet_ntoa(addrs[i]), stuff->bindings[i].server_tag);
 
 			res->status = NI_OK;
-			stat = RPC_SUCCESS;
 			goto binding_done;
 		}
+		else if (stat == RPC_SYSTEMERROR) break;
 	}
 
 	/*
@@ -2497,12 +2454,7 @@ bind_to_parent(struct in_addr *addrs, ni_rparent_stuff *stuff, unsigned int nadd
 					stuff->bindings[i].server_tag);
 		}
 
-		stat = ni_multi_call(nlocal, addrs,
-			NIBIND_PROG, NIBIND_VERS, NIBIND_BIND,
-			xdr_nibind_bind_args, stuff->bindings, 
-			sizeof(nibind_bind_args),
-			xdr_void, stuff, 
-			catch, -1);
+		stat = ni_multi_call(nlocal, addrs, NIBIND_PROG, NIBIND_VERS, NIBIND_BIND, (void *)xdr_nibind_bind_args, stuff->bindings, sizeof(nibind_bind_args), (void *)xdr_void, stuff, catch, -1);
 	}
 
 	if (stat != RPC_SUCCESS && nnetwork > 0)
@@ -2521,12 +2473,7 @@ bind_to_parent(struct in_addr *addrs, ni_rparent_stuff *stuff, unsigned int nadd
 					stuff->bindings[i].server_tag);
 		}
 
-		stat = ni_multi_call(nnetwork, addrs,
-			NIBIND_PROG, NIBIND_VERS, NIBIND_BIND,
-			xdr_nibind_bind_args, stuff->bindings, 
-			sizeof(nibind_bind_args),
-			xdr_void, stuff, 
-			catch, -1);
+		stat = ni_multi_call(nnetwork, addrs, NIBIND_PROG, NIBIND_VERS, NIBIND_BIND, (void *)xdr_nibind_bind_args, stuff->bindings, sizeof(nibind_bind_args), (void *)xdr_void, stuff,  catch, -1);
 	}
 
 	if (stat != RPC_SUCCESS && ((naddrs - nbcast) > 0))
@@ -2545,12 +2492,7 @@ bind_to_parent(struct in_addr *addrs, ni_rparent_stuff *stuff, unsigned int nadd
 					stuff->bindings[i].server_tag);
 		}
 
-		stat = ni_multi_call(naddrs, addrs,
-			NIBIND_PROG, NIBIND_VERS, NIBIND_BIND,
-			xdr_nibind_bind_args, stuff->bindings, 
-			sizeof(nibind_bind_args),
-			xdr_void, stuff, 
-			catch, -1);
+		stat = ni_multi_call(naddrs, addrs, NIBIND_PROG, NIBIND_VERS, NIBIND_BIND, (void *)xdr_nibind_bind_args, stuff->bindings, sizeof(nibind_bind_args), (void *)xdr_void, stuff, catch, -1);
 	}
 
 binding_done:
@@ -2664,13 +2606,13 @@ _ni_rparent_2_svc(void *arg, struct svc_req *req)
 	 */
 	if (alert_aborted())
 	{
-		set_binding_status(NI_NETROOT);
+		set_binding_status(NI_NETROOT, 0);
 		return &res;
 	}
 
 	if (sys_is_standalone())
 	{
-		set_binding_status(NI_NETROOT);
+		set_binding_status(NI_NETROOT, 0);
 		return &res;
 	}
 
@@ -2684,7 +2626,7 @@ _ni_rparent_2_svc(void *arg, struct svc_req *req)
 	 */
 	if (res.status == NI_OK)
 	{
-		if (ni_ping(paddr, res.ni_rparent_res_u.binding.tag))
+		if (ni_ping(paddr, res.ni_rparent_res_u.binding.tag) == RPC_SUCCESS)
 		{
 			/*
 			 * The address is stuffed into an unsigned long, 
@@ -2720,7 +2662,7 @@ _ni_rparent_2_svc(void *arg, struct svc_req *req)
 		/*
 		 * Set status to NI_NORESPONSE while we are trying to bind. 
 		 */
-		set_binding_status(NI_NORESPONSE);
+		set_binding_status(NI_NORESPONSE, 0);
 
 		/*
 		 * If there is a binding file, get potential parent servers from it.
@@ -2736,7 +2678,7 @@ _ni_rparent_2_svc(void *arg, struct svc_req *req)
 		{
 			/* Can't get any potential parents! */
 			res.status = NI_NETROOT;
-			set_binding_status(NI_NETROOT);
+			set_binding_status(NI_NETROOT, 0);
 			return &res;
 		}
 
@@ -2767,7 +2709,7 @@ _ni_rparent_2_svc(void *arg, struct svc_req *req)
 		time(&root_time);
 	}
 
-	set_binding_status(res.status);
+	set_binding_status(res.status, 0);
 
 	return (&res);
 }
@@ -3042,7 +2984,7 @@ _ni_readall_2_svc(
 				 * to unlock it, we just sit forever, eating up bits
 				 * of CPU time.
 				 */
-				didit = svc_sendreply(req->rq_xprt, readall, db_ni);
+				didit = svc_sendreply(req->rq_xprt, (void *)readall, db_ni);
 				if (!didit)
 				{
 					system_log(LOG_ERR, "readall %s to %s:%hu failed",
@@ -3128,7 +3070,7 @@ _ni_readall_2_svc(
 	 * of the proxy failed.  Do it here, regardless.
 	 */
 	socket_lock();
-	didit = svc_sendreply(req->rq_xprt, readall, db_ni);
+	didit = svc_sendreply(req->rq_xprt, (void *)readall, db_ni);
 	socket_unlock();
 	syslock_lock(readall_syslock);
 	sending_all--;	/* Let writing resume (well, once we're at 0) */
@@ -3284,6 +3226,106 @@ _ni_resync_2_svc(
 	return (&status);
 }
 
+/*
+ * Find the mapped port for program,version.
+ * Calls the pmap service remotely to do the lookup.
+ * Returns 0 if no map exists.
+ */
+static u_short
+ni_ping_pmap_getport(struct sockaddr_in *address, unsigned int program, unsigned int version, unsigned int protocol, int *err)
+{
+	u_short port = 0;
+	int sock = -1;
+	int errno_saved;
+	CLIENT *client;
+	struct pmap parms;
+	struct timeval tv, total;
+
+	tv.tv_sec = PING_TIMEOUT_SEC;
+	tv.tv_usec = PING_TIMEOUT_USEC;
+	
+	total.tv_sec = PING_CALL_SEC;
+	total.tv_usec = PING_CALL_USEC;
+	
+	socket_lock();
+	sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+	errno_saved = errno;
+	socket_unlock();
+	if (sock < 0)
+	{
+		syslog(LOG_DEBUG, "ni_ping_pmap_getport: couldn't get socket - %s", strerror(errno_saved));
+		*err = RPC_SYSTEMERROR;
+		return 0;
+	}
+
+	address->sin_port = htons(PMAPPORT);
+	client = clntudp_bufcreate(address, PMAPPROG, PMAPVERS, tv, &sock, RPCSMALLMSGSIZE, RPCSMALLMSGSIZE);
+	if (client == NULL)
+	{
+		syslog(LOG_DEBUG, "ni_ping_pmap_getport: %s", clnt_spcreateerror("clntudp_bufcreate"));
+		socket_lock();
+		close(sock);
+		socket_unlock();
+		*err = RPC_FAILED;
+		return 0;
+	}
+
+	parms.pm_prog = program;
+	parms.pm_vers = version;
+	parms.pm_prot = protocol;
+	parms.pm_port = 0;  /* not needed or used */
+
+	*err = CLNT_CALL(client, PMAPPROC_GETPORT, (void *)xdr_pmap, &parms, (void *)xdr_u_short, &port, total);
+	if ((*err == RPC_SUCCESS) && (port == 0)) *err = RPC_PROGNOTREGISTERED;
+
+	socket_lock();
+	clnt_destroy(client);
+	close(sock);
+	socket_unlock();
+
+	address->sin_port = 0;
+
+	return port;
+}
+
+int
+ni_ping_socket_open(struct sockaddr_in *raddr, int prog, int vers, int *err)
+{
+	int sock;
+	int errno_saved;
+	u_short port;
+
+	/*
+	 * If no port number given ask the pmap for one
+	 */
+	if (raddr->sin_port == 0)
+	{
+		port = ni_ping_pmap_getport(raddr, prog, vers, IPPROTO_UDP, err);
+		if (port == 0)
+		{
+			syslog(LOG_DEBUG, "ni_ping_socket_open: %s", clnt_spcreateerror("pmap_getport failed"));
+			return -1;
+		}
+
+		raddr->sin_port = htons(port);
+	}
+	
+	socket_lock();
+	sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+	errno_saved = errno;
+	socket_unlock();
+
+	if (sock < 0)
+	{
+		syslog(LOG_WARNING, "ni_ping_socket_open: couldn't get socket - %s", strerror(errno_saved));
+		*err = RPC_SYSTEMERROR;
+		return -1;
+	}
+
+	*err = RPC_SUCCESS;
+	return sock;
+}
+
 /*
  * Ping the server at the given address/tag
  */
@@ -3293,7 +3335,7 @@ ni_ping(u_long address, ni_name tag)
 	struct sockaddr_in sin;
 	struct timeval tv, total;
 	enum clnt_stat stat;
-	int sock;
+	int sock, err;
 	nibind_getregister_res res;
 	CLIENT *cl;
 
@@ -3301,46 +3343,50 @@ ni_ping(u_long address, ni_name tag)
 	sin.sin_port = 0;
 	sin.sin_addr.s_addr = address;
 	bzero(sin.sin_zero, sizeof(sin.sin_zero));
-	sock = socket_open(&sin, NIBIND_PROG, NIBIND_VERS);
-	if (sock < 0) return 0;
+	err = RPC_SUCCESS;
+
+	sock = ni_ping_socket_open(&sin, NIBIND_PROG, NIBIND_VERS, &err);
+	if (sock < 0) return err;
 
 	tv.tv_sec = PING_TIMEOUT_SEC;
 	tv.tv_usec = PING_TIMEOUT_USEC;
-
+	
 	total.tv_sec = PING_CALL_SEC;
 	total.tv_usec = PING_CALL_USEC;
-
+	
 	cl = clntudp_create(&sin, NIBIND_PROG, NIBIND_VERS, tv, &sock);
 	if (cl == NULL)
 	{
+		syslog(LOG_DEBUG, "ni_ping: %s", clnt_spcreateerror("clntudp_create"));
 		socket_close(sock);
-		return 0;
+		return RPC_FAILED;
 	}
 
-	stat = clnt_call(cl, NIBIND_GETREGISTER, xdr_ni_name, &tag, xdr_nibind_getregister_res, &res, total);
+	stat = clnt_call(cl, NIBIND_GETREGISTER, (void *)xdr_ni_name, &tag, (void *)xdr_nibind_getregister_res, &res, total);
 	clnt_destroy(cl);
 	socket_close(sock);
 
-	if (stat != RPC_SUCCESS) return 0;
-	if (res.status != NI_OK) return 0;
+	if (stat != RPC_SUCCESS) return stat;
+	if (res.status != NI_OK) return RPC_PROGNOTREGISTERED;
 
 	/*
 	 * Actually talk to parent during ni_ping
 	 */
 	sin.sin_port = htons(res.nibind_getregister_res_u.addrs.udp_port);
 	sock = socket_open(&sin, NI_PROG, NI_VERS);
-	if (sock < 0) return 0;
+	if (sock < 0) return RPC_SYSTEMERROR;
 
 	cl = clntudp_create(&sin, NI_PROG, NI_VERS, tv, &sock);
 	if (cl == NULL)
 	{
+		syslog(LOG_DEBUG, "ni_ping: %s", clnt_spcreateerror("clntudp_create"));
 		socket_close(sock);
-		return 0;
+		return RPC_FAILED;
 	}
 
-	stat = clnt_call(cl, _NI_PING, xdr_void, (void *)NULL, xdr_void, (void *)NULL, total);
+	stat = clnt_call(cl, _NI_PING, (void *)xdr_void, (void *)NULL, (void *)xdr_void, (void *)NULL, total);
 	clnt_destroy(cl);
 	socket_close(sock);
 
-	return (stat == RPC_SUCCESS);
+	return stat;
 }
