@@ -23,11 +23,12 @@
  */
 
 #include <stdlib.h>
-#include <syslog.h>
+#include <asl.h>
 #include <dirent.h>
 #include <sys/param.h>
 #include <sys/event.h>
 #include <sys/fcntl.h>
+#include <unistd.h>
 #include "file_watcher.h"
 #include "daemon.h"
 
@@ -90,7 +91,7 @@ file_watcher_add_kq(file_watcher_t *f)
 	if (f->kqident != IDENT_INVAL) return;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_add_kq(%u, %s)", f->w->wid, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_add_kq(%u, %s)", f->w->wid, f->path);
 #endif
 
 	event.ident = open(f->path, O_EVTONLY, 0);
@@ -122,7 +123,7 @@ file_watcher_remove_kq(file_watcher_t *f)
 	if (f->kqident == IDENT_INVAL) return;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_remove_kq(%u, %s)", f->w->wid, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_remove_kq(%u, %s)", f->w->wid, f->path);
 #endif
 
 	event.ident = f->kqident;
@@ -135,7 +136,7 @@ file_watcher_remove_kq(file_watcher_t *f)
 	status = kevent(kq, &event, 1, NULL, 0, NULL);
 	if (status != 0)
 	{
-		log_message(LOG_ERR, "EV_DELETE failed for file watcher %u", f->w->wid);
+		log_message(ASL_LEVEL_ERR, "EV_DELETE failed for file watcher %u", f->w->wid);
 	}
 
 	close(f->kqident);
@@ -286,7 +287,7 @@ file_watcher_update_dir(file_watcher_t *f, w_event_t **delta)
 	uint32_t do_notify;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_update_dir(%u, %s)", f->w->wid, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_update_dir(%u, %s)", f->w->wid, f->path);
 #endif
 
 	dp = opendir(f->path);
@@ -408,7 +409,7 @@ file_watcher_remove(file_watcher_t *f)
 	char *pdir, *p;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_remove(%u, %s)", f->w->wid, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_remove(%u, %s)", f->w->wid, f->path);
 #endif
 
 	if (f == NULL) return 0;
@@ -483,7 +484,7 @@ file_watcher_remove(file_watcher_t *f)
 	}
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_remove %s - removed", f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_remove %s - removed", f->path);
 #endif
 
 	f->ftype = FS_TYPE_NONE;
@@ -501,7 +502,7 @@ file_watcher_update_link(file_watcher_t *f, w_event_t **delta)
 	file_watcher_t *t;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_update_link(%u, %s)", f->w->wid, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_update_link(%u, %s)", f->w->wid, f->path);
 #endif
 
 	n = readlink(f->path, lpath, MAXPATHLEN + 1);
@@ -666,7 +667,7 @@ file_watcher_update(file_watcher_t *f, uint32_t flags, uint32_t level)
 	int32_t status, i, do_notify, did_remove;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_update(%u, 0x%08x, %s)", f->w->wid, flags, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_update(%u, 0x%08x, %s)", f->w->wid, flags, f->path);
 #endif
 
 	if (f == NULL) return 0;
@@ -808,7 +809,7 @@ file_watcher_new(const char *p)
 	if (w == NULL) return NULL;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_new(%u, %s)", w->wid, p);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_new(%u, %s)", w->wid, p);
 #endif
 
 	f = (file_watcher_t *)calloc(1, sizeof(file_watcher_t));
@@ -843,7 +844,7 @@ file_watcher_new(const char *p)
 	file_watcher_update(f, 0, 0);
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_new: %s type %u trigger %u", p, f->ftype, w->wid);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_new: %s type %u trigger %u", p, f->ftype, w->wid);
 #endif
 
 	return w;
@@ -859,7 +860,7 @@ file_watcher_trigger(watcher_t *w, uint32_t flags, uint32_t level)
 	f = (file_watcher_t *)w->sub;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_trigger: %s type %u wid %u flags 0x%08x", f->path, f->ftype, w->wid, flags);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_trigger: %s type %u wid %u flags 0x%08x", f->path, f->ftype, w->wid, flags);
 #endif
 
 	return file_watcher_update(f, flags, level);
@@ -877,7 +878,7 @@ file_watcher_free(watcher_t *w)
 	f = (file_watcher_t *)w->sub;
 
 #ifdef DEBUG
-	log_message(LOG_ERR, "file_watcher_free(%u, %s)", f->w->wid, f->path);
+	log_message(ASL_LEVEL_DEBUG, "file_watcher_free(%u, %s)", f->w->wid, f->path);
 #endif
 
 	if (f->path != NULL) free(f->path);
